#include "Main.h"

int release(void);
int acquire(void);

// The static singleton
CGraphicsProcessor* CGraphicsProcessor::s_theGs = (CGraphicsProcessor*) NULL;

CGraphicsProcessor::CGraphicsProcessor( int nInterlace   /* = PS2_GS_NOINTERLACE */,
                                        int nTVOutMode   /* = PS2_GS_NTSC        */,
                                        int nFFmode		/* = PS2_GS_FRAME       */,
                                        int nResolution	/* = PS2_GS_640x480     */,
                                        int nRefreshRate /* = PS2_GS_60Hz        */,
                                        int nPsm			/* = PS2_GS_PSMCT32     */,
                                        int nZPsm			/* = PS2_GS_PSMZ24      */,
                                        int nColorDepth  /* = 24 */ )
{
   assert( s_theGs == NULL ); // this is a singleton !
   s_theGs = this;

   m_gp = NULL;

   m_nFileDescriptor = -1;

   memset( &m_db, 0, sizeof(ps2_gs_dbuff) );
   memset( &m_finish, 0, sizeof(ps2_gs_finish) );

   m_nInterlace   = nInterlace;
   m_nOutTVMode   = nTVOutMode;
   m_nFFmode      = nFFmode;
   m_nResolution  = nResolution;
   m_nRefreshRate = nRefreshRate;
   m_nPsm         = nPsm;
   m_nZPsm        = nZPsm;
   m_nColorDepth  = nColorDepth;

   m_nNext2k   = 0;
   m_nTextop64 = 0;

   m_nFrame = 0;
   m_nOdev  = 0;

   memset( &m_teTexture, 0, sizeof(m_teTexture) );	
   memset( &m_img, 0, sizeof(m_img) );
}

CGraphicsProcessor::~CGraphicsProcessor()
{
   s_theGs == NULL;
}

void CGraphicsProcessor::Open()
{
   // Get sys info from library
   m_gp = ps2_gs_get_gparam();

   m_nFileDescriptor = ps2_gs_open(-1);

   // ???
	ps2_gs_vc_graphicsmode();	
	
   // Resets GS by spec. settings	
	Reset();

	// Sets data for d.buffering	
	m_nNext2k = ps2_gs_set_dbuff( &m_db, m_nPsm, m_gp->width, m_gp->height,	(m_nColorDepth == 0) ? 0 : PS2_GS_ZGREATER, m_nZPsm, 1);

   // Data = R,G,B, ALPHA, ?	
	*(__u64 *)&m_db.clear0.rgbaq = PS2_GS_SETREG_RGBAQ(0x10, 0x10, 0x10, 0x80,0x3f800000);
	
   // Data = R,G,B, ALPHA, ?	
	*(__u64 *)&m_db.clear1.rgbaq = PS2_GS_SETREG_RGBAQ(0x10, 0x10, 0x10, 0x80,0x3f800000);
	
	// ??
	m_nTextop64 = m_nNext2k * 2048 / 64;
						
   // Init drawing enviroment	
	ps2_gs_put_drawenv(&m_db.giftag1);
												
   // Clear back buffer	
	ps2_gs_set_finish(&m_finish);
	
   // Wait until clear done										
	ps2_gs_wait_finish(&m_finish);
	
   // Sync with V-Blank	
	m_nOdev = ! ps2_gs_sync_v(0);
	
   // Start / Stop display	
	ps2_gs_start_display(1);

   // ************ ACQ - REL												
	ps2_gs_vc_enablevcswitch( acquire, release );						
}

void CGraphicsProcessor::Reset()
{	
   if ( IsOpen() )
   	ps2_gs_reset( 0, m_nInterlace, m_nOutTVMode, m_nFFmode, m_nResolution, m_nRefreshRate );	
}

void CGraphicsProcessor::Close()
{
	if ( IsOpen() )
	{
		ps2_gs_close();
		m_nFileDescriptor = -1;
	}	

	m_vp.Close();
}

void CGraphicsProcessor::PrepareDisplay()
{
   ps2_gs_vc_lock();

   // Create draw offset area		
   ps2_gs_set_half_offset( (m_nFrame & 1) ? &m_db.draw1 : &m_db.draw0, m_nOdev );
		
   // Swap double buffer
   ps2_gs_swap_dbuff( &m_db, m_nFrame );							
	
   // Create data for text. settings
	ps2_gs_set_texenv( &m_teTexture.gs_tex, 0, &m_img, 8, 8, 0, 0, 0, 0, 1 );
		
   PS2_GIFTAG_CLEAR_TAG( &m_teTexture.giftag );						
   m_teTexture.giftag.NLOOP = sizeof(m_teTexture) / 16 - 1;				
   m_teTexture.giftag.EOP = 1;										
   m_teTexture.giftag.PRE = 0;										
   m_teTexture.giftag.NREG = 1;										
   m_teTexture.giftag.REGS0 = PS2_GIFTAG_REGS_AD;					
		
   // Prep. for alpha blending
   ps2_gs_set_alphaenv(&m_teTexture.gs_alpha, 0);					
		
   *(__u64 *)&m_teTexture.gs_alpha.alpha1 = PS2_GS_SETREG_ALPHA(0, 1, 0, 1, 0);
	
   m_teTexture.gs_test1addr = PS2_GS_TEST_1;
		
   *(__u64 *)&m_teTexture.gs_test = PS2_GS_SETREG_TEST(0, 0, 0, 0, 0, 0, 1, (m_nColorDepth == 0) ? PS2_GS_ZALWAYS : PS2_GS_ZGREATER);
		
   *(__u64 *)&m_teTexture.gs_tex.clamp1 = PS2_GS_SETREG_CLAMP(0, 0, 0, 0, 0, 0);
	
   // Set drawing enviroment
   ps2_gs_put_drawenv( &m_teTexture.giftag );
}

void CGraphicsProcessor::SyncV()
{
   m_nFrame++; // Counter for double buffer
   m_nOdev = ! ps2_gs_sync_v(0);
	ps2_gs_vc_unlock();
}

void CGraphicsProcessor::Draw( ps2_giftag* pGifTag )
{
   ps2_gs_put_drawenv( pGifTag );
}
	
void CVectorProcessorUnit::Open()
{
   assert( m_vpu0 == NULL );
   assert( m_vpu1 == NULL );
	
   m_vpu0 = ps2_vpu_open(0);
   m_vpu1 = ps2_vpu_open(1);
}

void CVectorProcessorUnit::Reset()
{
   if ( IsOpen() )
   {
      ps2_vpu_reset( m_vpu1 ); // reset vpu1 first
      ps2_vpu_reset( m_vpu0 );
   }
}

void CVectorProcessorUnit::Close()
{	
	if ( m_vpu1 )
	{
		ps2_vpu_close( m_vpu1 );
		m_vpu1 = NULL;
	}
	
	if ( m_vpu0 )
	{
		ps2_vpu_close( m_vpu0 );
		m_vpu0 = NULL;
	}
}

int acquire(void)
{
   CGraphicsProcessor* pGs = CGraphicsProcessor::GetGs();
   assert(pGs);

   CVectorProcessorUnit* pVp = CGraphicsProcessor::GetVp();
   assert(pVp);

   pVp->Open();

	if ( ! pGs->IsOpen() || ! pGs->IsOpen() )
	{
		release();
	   return PS2_GS_VC_ACQ_FAILURE;
	}
	
	pVp->Reset();
	pGs->Reset();
	
	return PS2_GS_VC_ACQ_SUCCESS;
}

int release(void)
{
	if ( CGraphicsProcessor::GetGs() && CGraphicsProcessor::GetGs()->IsOpen() )
	   CGraphicsProcessor::GetGs()->Close();
	
	return PS2_GS_VC_REL_SUCCESS;
}