#ifndef _CGraphicsProcessor_h_
#define _CGraphicsProcessor_h_

typedef struct														// Context1 texture info.
{
	ps2_giftag		 giftag;
	ps2_gsreg_test  gs_test;
	__u64			    gs_test1addr;
	ps2_gs_alphaenv gs_alpha;
	ps2_gs_texenv	 gs_tex;
} TexEnv;

class CVectorProcessorUnit
{
friend class CGraphicsProcessor;

public:
   CVectorProcessorUnit() { m_vpu0 = NULL; m_vpu1 = NULL; }

   void Open();
   void Reset();
   void Close();

public:
   BOOL IsOpen() { return m_vpu0 != NULL && m_vpu1 != NULL; }

protected:
   ps2_vpu* m_vpu0;
   ps2_vpu* m_vpu1;
};

class CGraphicsProcessor
{
public:
   // Constructor / destructor
   CGraphicsProcessor( int nInterlace   = PS2_GS_NOINTERLACE,
                       int nOutMode     = PS2_GS_NTSC,
                       int nFFmode		 = PS2_GS_FRAME,
                       int nResolution	 = PS2_GS_640x480,
                       int nRefreshRate = PS2_GS_60Hz,
                       int nPsm			 = PS2_GS_PSMCT32,
                       int nZPsm			 = PS2_GS_PSMZ24,
                       int nColorDepth  = 24 );
   virtual ~CGraphicsProcessor();

public:
   void Open();
   void Reset();
   void Close();

   void PrepareDisplay();
   void Draw( ps2_giftag* pGifTag );
   void SyncV();

   BOOL           IsOpen()   { return m_nFileDescriptor >= 0; }
   ps2_gs_gparam* GetGparm() { return m_gp; }

public:
static CGraphicsProcessor*   GetGs() { return s_theGs; }
static CVectorProcessorUnit* GetVp() { return( s_theGs? &(s_theGs->m_vp) : NULL ); }

private:
   ps2_gs_gparam* m_gp;			// Param's used by library
   ps2_gs_dbuff   m_db;			// Data for double buffering
   ps2_gs_finish  m_finish;   // Struct for GS drawing done
   ps2_image      m_img;      //

   int m_nInterlace;
   int m_nOutTVMode;
   int m_nFFmode;
   int m_nResolution;
   int m_nRefreshRate;
   int m_nPsm;
   int m_nZPsm;
   int m_nColorDepth;

   int m_nNext2k;
   int m_nTextop64;

   int m_nFrame;           // Used for d.buffered display
	int m_nOdev;				// Used in ps2_gs_sync_v
   int m_nFileDescriptor;

   // Struct object for tex. info.
   TexEnv m_teTexture;														

   CVectorProcessorUnit m_vp;

private:
   static CGraphicsProcessor* s_theGs;
};


#endif // _CGraphicsProcessor_h_