#include "Main.h"

short Round(double x)
{
  return((short)(x+0.5));
}

short Trunc(double x)
{
  return((short)(x));
}

float Frac(double x)
{
  short y;

  y=((short)(x));
  return(x-(float)y);
}

void VecNull(TDA A)
{
  A[0]=0.0;
  A[1]=0.0;
  A[2]=0.0;
}

float Radians(float Angle)
{
  return(Angle*PiOver180);
}

float SqrFP(float x)
{
  return(x*x);
}

float Log(float x)
{
  return(log(x)*OneOverLn10);
}

float MIN(float a, float b)
{
  if(a<b)
    return(a);
  else
    return(b);
}

float MAX(float a, float b)
{
  if(a>b)
    return(a);
  else
    return(b);
}

float MIN3(float a, float b, float c)
{
  float t;

  t=MIN(a, b);
  return(MIN(t, c));
}

float MAX3(float a, float b, float c)
{
  float t;

  t=MAX(a, b);
  return(MAX(t, c));
}

float MIN4(float a, float b, float c, float d)
{
  float t;

  t=MIN3(a, b, c);
  return(MIN(t, d));
}

float MAX4(float a, float b, float c, float d)
{
  float t;

  t=MAX3(a, b, c);
  return(MAX(t, d));
}


/* ***********************************************************************
   *									 *
   *			Vector And Matrix Routines			 *
   *									 *
   ***********************************************************************

   Vec             - Make Vector
   VecInt	   - Make Integer Vector
   UnVec	   - Get Components of vector
   UnVecInt	   - Get Components of Integer Vector
   VecDot	   - Vector Dot Product
   VecCross	   - Vector Cross Product
   VecLen	   - Vector Length
   VecNormalize	   - Vector Normalize
   VecMatxMult	   - Vector Matrix Multiply
   VecSub	   - Vector Subtraction
   VecSubInt	   - Vector Subtraction Integer
   VecAdd	   - Vector Addition
   VecAdd3	   - Vector Addition
   VecCopy	   - Vector Copy
   VecLinComb	   - Vector Linear Combination
   VecScalMult	   - Vector Scalar Multiple
   VecScalMultI    - Vector Scalar Multiple
   VecScalMultInt  - Vector Scalar Multiple and Rounding
   VecAddScalMult  - Vector Add Scalar Multiple
   VecNull	   - Vector Null
   VecNullInt	   - Vector Null Integer
   VecElemMult	   - Vector Element Multiply
*/

void Vec(float r, float s, float t, TDA A)
{
  A[0]=r;
  A[1]=s;
  A[2]=t;
}

void UnVec(TDA A, float *r, float *s, float *t)
{
  *r=A[0];
  *s=A[1];
  *t=A[2];
}

void VecCross(TDA A, TDA B, TDA C)
{
  C[0]=A[1]*B[2] - A[2]*B[1];
  C[1]=A[2]*B[0] - A[0]*B[2];
  C[2]=A[0]*B[1] - A[1]*B[0];
}

float VecLen(TDA A)
{
  return(sqrt(SqrFP(A[0])+SqrFP(A[1])+SqrFP(A[2])));
}

void VecNormalize(TDA A)
{
  float dist, invdist;

  dist=VecLen(A);
  if(!(dist==0.0))
  {
    invdist=1.0/dist;
    A[0]*=invdist;
    A[1]*=invdist;
    A[2]*=invdist;
  }
  else
  {
    puts("Zero-Length Vectors cannot be Normalized");
    exit(1);
  }
}

float VecDot(TDA A, TDA B)
{
  return(A[0]*B[0] + A[1]*B[1] + A[2]*B[2]);
}

void VecSub(TDA A, TDA B, TDA C)
{
  C[0]=A[0]-B[0];
  C[1]=A[1]-B[1];
  C[2]=A[2]-B[2];
}

void VecCopy(TDA A, TDA B)
{
  B[0]=0.0+A[0];
  B[1]=0.0+A[1];
  B[2]=0.0+A[2];
}

void VecAdd(TDA A, TDA B, TDA C)
{
  C[0]=A[0]+B[0];
  C[1]=A[1]+B[1];
  C[2]=A[2]+B[2];
}

void VecAdd3(TDA A, TDA B, TDA C, TDA D)
{
  D[0]=A[0]+B[0]+C[0];
  D[1]=A[1]+B[1]+C[1];
  D[2]=A[2]+B[2]+C[2];
}

void VecScalMult(float r, TDA A, TDA B)
{
  B[0]=r*A[0];
  B[1]=r*A[1];
  B[2]=r*A[2];
}

void VecScalMultI(float r, TDIA A, TDA B)
{
  B[0]=r*(float)A[0];
  B[1]=r*(float)A[1];
  B[2]=r*(float)A[2];
}

void VecScalMultInt(float r, TDA A, TDIA B)
{
  B[0]=Round(r*A[0]);
  B[1]=Round(r*A[1]);
  B[2]=Round(r*A[2]);
}

void VecMin(TDA a, TDA b, TDA c)
{
   if ( a[0]<b[0] )
      c[0]=a[0];
   else
      c[0]=b[0];

   if ( a[1]<b[1] )
      c[1]=a[1];
   else
      c[1]=b[1];

   if( a[2]<b[2] )
      c[2]=a[2];
   else
      c[2]=b[2];
}

void VecMax(TDA a, TDA b, TDA c)
{
   if ( a[0]>b[0] )
      c[0]=a[0];
   else
      c[0]=b[0];

   if ( a[1]>b[1] )
      c[1]=a[1];
   else
      c[1]=b[1];

   if ( a[2]>b[2] )
      c[2]=a[2];
   else
      c[2]=b[2];
}

void VecNegate(TDA A)
{
  A[0]=-A[0];
  A[1]=-A[1];
  A[2]=-A[2];
}

void VecLinComb(float r, TDA A, float s, TDA B, TDA C)
{
  C[0]=r*A[0]+s*B[0];
  C[1]=r*A[1]+s*B[1];
  C[2]=r*A[2]+s*B[2];
}

void VecAddScalMult(float r, TDA A, TDA B, TDA C)
{
  C[0]=r*A[0]+B[0];
  C[1]=r*A[1]+B[1];
  C[2]=r*A[2]+B[2];
}

void VecElemMult(float r, TDA A, TDA B, TDA C)
{
  C[0]=r*A[0]*B[0];
  C[1]=r*A[1]*B[1];
  C[2]=r*A[2]*B[2];
}


